#ifndef _FILELOCK_H
#define _FILELOCK_H

/**
 * @file filelock.h
 */
#include <sys/types.h>
#include <fcntl.h>

#ifdef	__cplusplus
extern "C" {
#endif


/**
 * @brief This was taken from APUE p. 370.  Used for file record locking.
 * Pass offset as 0 and whence to SEEK_SET to lock entire file.
 *
 * @param fd - file descriptor
 * @param cmd -  F_GETLK, F_SETLK, F_SETLKW ('W' signifies wait)
 * @param type - F_RDLCK, R_WRLCK, F_UNLCK
 * @param offset -byte offset, relative to whence
 * @param whence- SEEK_SET, SEEK_CUR, SEEK_END
 * @param len - # bytes (0 means to EOF)
 * @return -1 on error
 */
int filelock_region (int fd, int cmd, int type, off_t offset, int whence,
			off_t len);

/**
 * @brief filelock_getrlock() used to obtain advisory READ lock on filename.
 * The file descriptor is returned to caller.  Caller must specify
 * if they want to wait for the lock by setting wait4lock.
 *
 * @note  caller does not have to close the file or call unlock()
 * routine when this routine returns FALSE.
 *
 * @param filename
 * @param fd
 * @param wait4lock
 * @return
 */
int filelock_getrlock (char *filename, int *fd, int wait4lock);

/**
 * @brief filelock_getlock() used to obtain advisory WRITE lock on filename.
 * The file descriptor is returned to caller.  Caller must specify
 * if they want to wait for the lock by setting wait4lock.
 *
 * @note  caller does not have to close the file or call unlock()
 * routine when this routine returns FALSE.
 *
 * @param filename
 * @param fd
 * @param wait4lock
 * @return
 */
int filelock_getlock (char *filename, int *fd, int wait4lock);

/**
 * @brief filelock_unlock() is used to release the advisory locked from a prior
 * call to filelock_getlock().  The file is also closed after the lock
 * is released.
 *
 * @param fd - file descriptor
 * @return
 */
int filelock_unlock (int fd);

#ifdef	__cplusplus
}
#endif

#endif
