/**
 * @fn xmlconftable.h
 * @brief define the public routines and data used by the xml2table tool
 * the tool is used to take an xml file and build a table of records where
 * each table record is built from arrtibutes in an xml tag. 
 * each tag and attributes should have the format
 * 
 * <tagname att1="..." att2="..." ... attn="..." />
 * 
 * that is the tag defines the record, the attributes are the fields in the
 * record. each field is a string of characters, no conversion to numerics is
 * done. the table is an array of pointers to structs that contain pointers 
 * to the strings from the attibutes in the file.  
 * 
 * file example:
 * 
 * <?xml version="1.0" encoding="utf-8"?>
 *   <codemap>  
 *     <item code="Aerospace\Aircraft" short="Aircraft" type="SU" />
 *     <item code="Electronics and Communications" short="Elect and Comm" type="SU" />
 *     ...
 *   </codemap>
 * 
 * the item tags will be formed into a table of records  with fields made from
 * the attributes in the tags.
 * 
 * NOTE:  the first field, in the RecordDef, is considered the key field.  
 * it need not be the first field in the xml attributes but it must be the 
 * first field in the 'record definition' structure passed to the function to
 * create the table.
 *   
 * Also all searches against the key fields are done using a non case sensitive
 * compare so make note of that !!!
 * 
 * If one passes a 'record definition' struct that has a field countof 5, 
 * there better be 5 field names pointed to by the fieldnames pointer!
 * 
 * there is currently no way to deallocate a table. sorry....  a failure to 
 * build a table tries to deallocate any memory that was allocated during 
 * the process that failed. 
 *
 * a special search function is made available so that you can implement
 * what ever you would like to do if a find operation on the key you passed
 * in fails. you should return NULL if your special search fails or a pointer
 * to a record of pointers just like the find operation does.  this has been
 * used to implement an unknown functionality, if the find fails the callback
 * can return a 'fake' record with values that indicate unknown
 *
*/

#if !defined __xml_2_table__ 
#define __xml_2_table__
/* ------------------------------------------------------------------------ */
#if defined __cplusplus
extern "C" {
#endif

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
/** 
 *   @struct RecordDef
 *   @brief structure passed to define the table to be built.  
*/
typedef struct __record_definition__ {
    const char *tag_name;  //!< @brief ptr name of tag used to define a record
    size_t fieldcount;  //!< @brief count of  attributes for fields
    const char **fieldnames;  //!< @brief array of att  names to look for, 0th is key field
} XCT_RecordDef;

typedef void *XCThandle;  //!< typedef for 'handle' to the table 

/** specialsearch 
 *  @brief a pointer to a funciton that will be called if the normal search
 *  fails. it is called with the key that was being searched for and a pointer
 *  to the table and the count of records, rows in the table.
 *  @param conts char * key  the value of the key used to do the search that 
 *  failed
 *  @param cont void * table  the array of pointers to the records in the 
 *  table you probably want to cast this to a const char ***table and use it
 *  like  table[r][1...n] where r is the row and 1 ... n are the fields in
 *  the record def order.
 *  @retvval  you need to return either NULL to pass failure back to the 
 *  call that did the find or a pointer to an array of const char * that 
 *  match the fields in the records return on a successful search.  
*/

typedef const char ** (*XCTSpecialSearch) ( const char * key, const void *table, size_t tablecount);

/** @fn X2T_Init
 * @brief init routine. called to build the table from the contents of the file.
 * @param const char * path2file
 * @param const RecordDef * recdef  record definition for this file/table
 * @param pointer to a special search function to be called if the standard find 
 * fails or a NULL to indicate no special search callback is wanted. 
 * @retval X2Thandle or 0  if 0 is returned there was an error.
*/

XCThandle XCT_Init(const char *path2file, const XCT_RecordDef *recdef, XCTSpecialSearch SS_fn);

/** @fn X2T_Find 
 * @brief the routine to call to map a code.
 * @param X2thandle handle to the table you want to search
 * @param const char *key the key field to find
 * @retval const char ** pointer to field values in the fourn record
 * returns NULL when no record with the key was found. the incoming key is
 * matched against the first field of the row.
*/

const char **XCT_Find(XCThandle t, const char *key);  /* contents of the key field to find */

/* ------------------------------------------------------------------------ */
/** #fn X2T_GetTable
 * @brief  so you want to see what the table looks like after it was pulled 
 * in from the file...  this gives you that. cast the returned pointer to a
 * const char *** and then reference frow and fields this way...
 * pTable[row][field]
 * returns count of entries in use.  puts base of the table at *outtable
 * @param X2Thandle t   handle of the table you want to look at
 * @param void ** table   pointer to table of pointers to the rows
 * @retval count of table rows
*/

size_t XCT_GetTable(XCThandle t, void **table);

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
#if defined __cplusplus
}
#endif
/* ------------------------------------------------------------------------ */
#endif /* if !defined __xml_2_table__  */
